@@ -6,12 +6,14 @@
 #include<core/State.hpp>
 #include<pkg/common/Sphere.hpp>
 
+namespace yade { // Cannot have #include directive inside.
+
 YADE_PLUGIN((Clump));
 CREATE_LOGGER(Clump);
 
 boost::python::dict Clump::members_get(){
   boost::python::dict ret;
-	FOREACH(MemberMap::value_type& b, members){
+	for(const auto & b : members){
 		ret[b.first]=boost::python::make_tuple(b.second.position,b.second.orientation);
 	}
 	return ret;
@@ -24,7 +26,7 @@ void Clump::add(const shared_ptr<Body>& clumpBody, const shared_ptr<Body>& subBo
 	if(subBody->isClumpMember()) throw std::invalid_argument(("Body #"+boost::lexical_cast<string>(subId)+" is already a clump member of #"+boost::lexical_cast<string>(subBody->clumpId)).c_str());
 	else if(subBody->isClump()){
 		const shared_ptr<Clump> subClump=YADE_PTR_CAST<Clump>(subBody->shape);
-		FOREACH(const MemberMap::value_type& mm, subClump->members){
+		for (const auto & mm : subClump->members){
 			const Body::id_t& memberId=mm.first;
 			Scene* scene(Omega::instance().getScene().get());	// get scene
 			const shared_ptr<Body>& member=Body::byId(memberId,scene);
@@ -53,7 +55,7 @@ void Clump::del(const shared_ptr<Body>& clumpBody, const shared_ptr<Body>& subBo
 }
 
 void Clump::addForceTorqueFromMembers(const State* clumpState, Scene* scene, Vector3r& F, Vector3r& T){
-	FOREACH(const MemberMap::value_type& mm, members){
+	for (const auto & mm : members){
 		const Body::id_t& memberId=mm.first; 
 		const shared_ptr<Body>& member=Body::byId(memberId,scene); 
 		assert(member->isClumpMember()); 
@@ -114,10 +116,10 @@ void Clump::updateProperties(const shared_ptr<Body>& clumpBody, unsigned int dis
 	shared_ptr<Sphere> sph (new Sphere);
 	int Sph_Index = sph->getClassIndexStatic();		// get sphere index for checking if bodies are spheres
 	if (discretization>0){
-		FOREACH(MemberMap::value_type& mm, clump->members){
+		for (const auto & mm : clump->members){
 			const shared_ptr<Body> subBody1=Body::byId(mm.first);
-			FOREACH(MemberMap::value_type& mm, clump->members){
-				const shared_ptr<Body> subBody2=Body::byId(mm.first);
+			for (const auto & mmm : clump->members){
+				const shared_ptr<Body> subBody2=Body::byId(mmm.first);
 				if ((subBody1->shape->getClassIndex() ==  Sph_Index) && (subBody2->shape->getClassIndex() ==  Sph_Index) && (subBody1!=subBody2)){//clump members should be spheres
 					Vector3r dist = subBody1->state->pos - subBody2->state->pos;
 					const Sphere* sphere1 = YADE_CAST<Sphere*> (subBody1->shape.get());
@@ -146,7 +148,7 @@ void Clump::updateProperties(const shared_ptr<Body>& clumpBody, unsigned int dis
 	if(intersecting){	
 		//get boundaries of clump:
 		AlignedBox3r aabb;
-		FOREACH(MemberMap::value_type& mm, clump->members){
+		for ( const  auto & mm : clump->members){
 			const shared_ptr<Body> subBody = Body::byId(mm.first);
 			if (subBody->shape->getClassIndex() == Sph_Index){//clump member should be a sphere
 				const Sphere* sphere = YADE_CAST<Sphere*> (subBody->shape.get());
@@ -159,13 +161,13 @@ void Clump::updateProperties(const shared_ptr<Body>& clumpBody, unsigned int dis
 		//get volume and inertia tensor using regular cubic cell array inside bounding box of the clump:
 		Real dx = rMin/discretization; 	//edge length of cell
 		Real dv = pow(dx,3);		//volume of cell
-		long nCells=(aabb.sizes()/dx).prod();
+		long nCells=long(std::round((aabb.sizes()/dx).prod()));
 		if(nCells>1e7) LOG_WARN("Clump::updateProperties: Cell array has "<<nCells<<" cells. Integrate inertia may take a while ...");
 		Vector3r x;			//position vector (center) of cell
 		for(x.x()=aabb.min().x()+dx/2.; x.x()<aabb.max().x(); x.x()+=dx){
 			for(x.y()=aabb.min().y()+dx/2.; x.y()<aabb.max().y(); x.y()+=dx){
 				for(x.z()=aabb.min().z()+dx/2.; x.z()<aabb.max().z(); x.z()+=dx){
-					FOREACH(MemberMap::value_type& mm, clump->members){
+					for ( const  auto & mm : clump->members){
 						const shared_ptr<Body> subBody = Body::byId(mm.first);
 						if (subBody->shape->getClassIndex() == Sph_Index){//clump member should be a sphere
 							dens = subBody->material->density;
@@ -175,7 +177,7 @@ void Clump::updateProperties(const shared_ptr<Body>& clumpBody, unsigned int dis
 								M += m;
 								Sg += m*x;
 								//inertia I = sum_i( mass_i*dist^2 + I_s) )	//steiners theorem
-								Ig += m*( x.dot(x)*Matrix3r::Identity()-x*x.transpose())/*dist^2*/+Matrix3r(Vector3r::Constant(dv*pow(dx,2)/6.).asDiagonal())/*I_s/m = d^2: along princial axes of dv; perhaps negligible?*/;
+								Ig += m*( x.dot(x)*Matrix3r::Identity()-x*x.transpose()/*dist^2*/+Matrix3r(Vector3r::Constant(pow(dx,2)/6.).asDiagonal()))/*I_s/m = d^2: along princial axes of dv; perhaps negligible?*/;
 								break;
 							}
 						}
@@ -184,7 +186,7 @@ void Clump::updateProperties(const shared_ptr<Body>& clumpBody, unsigned int dis
 			}
 		}
 	} else {//not intersecting
-		FOREACH(MemberMap::value_type& mm, clump->members){
+		for ( const  auto & mm : clump->members){
 			// mm.first is Body::id_t, mm.second is Se3r of that body
 			const shared_ptr<Body> subBody=Body::byId(mm.first);
 			dens = subBody->material->density;
@@ -229,7 +231,7 @@ void Clump::updateProperties(const shared_ptr<Body>& clumpBody, unsigned int dis
 	clumpBody->setAspherical(state->inertia[0]!=state->inertia[1] || state->inertia[0]!=state->inertia[2]);
 
 	// update subBodySe3s; subtract clump orientation (=apply its inverse first) to subBody's orientation
-	FOREACH(MemberMap::value_type& I, clump->members){
+	for (auto & I : clump->members){
 		shared_ptr<Body> subBody=Body::byId(I.first);
 		State* subState=subBody->state.get();
 		I.second.orientation=state->ori.conjugate()*subState->ori;
@@ -270,7 +272,7 @@ void Clump::updatePropertiesNonSpherical(const shared_ptr<Body>& clumpBody, bool
 		s: local subBody's coordinates
 		c: local clump coordinates
 	*/
-	double M=0; // mass
+	Real M=0; // mass
 	Vector3r Sg(0,0,0); // static moment, for getting clump's centroid
 	Matrix3r Ig(Matrix3r::Zero()), Ic(Matrix3r::Zero()); // tensors of inertia; is upper triangular, zeros instead of symmetric elements
 
@@ -281,7 +283,7 @@ void Clump::updatePropertiesNonSpherical(const shared_ptr<Body>& clumpBody, bool
 
 	// begin non-intersecting loop here
 	if(!intersecting){
-		FOREACH(MemberMap::value_type& I, clump->members){
+		for (const auto & I : clump->members){
 			// I.first is Body::id_t, I.second is Se3r of that body
 			shared_ptr<Body> subBody=Body::byId(I.first,rb);
 			State* subState=subBody->state.get();
@@ -333,7 +335,7 @@ void Clump::updatePropertiesNonSpherical(const shared_ptr<Body>& clumpBody, bool
 	clumpBody->setAspherical(state->inertia[0]!=state->inertia[1] || state->inertia[0]!=state->inertia[2]);
 
 	// update subBodySe3s; subtract clump orientation (=apply its inverse first) to subBody's orientation
-	FOREACH(MemberMap::value_type& I, clump->members){
+	for (auto & I : clump->members){
 		// now, I->first is Body::id_t, I->second is Se3r of that body
 		shared_ptr<Body> subBody=Body::byId(I.first,rb);
 		//const shared_ptr<RigidBodyParameters>& subRBP(YADE_PTR_CAST<RigidBodyParameters>(subBody->physicalParameters));
@@ -374,7 +376,7 @@ void Clump::updatePropertiesNonSpherical(const shared_ptr<Body>& clumpBody, bool
 		s: local subBody's coordinates
 		c: local clump coordinates
 	*/
-	double M=0; // mass
+	Real M=0; // mass
 	Vector3r Sg(0,0,0); // static moment, for getting clump's centroid
 	Matrix3r Ig(Matrix3r::Zero()), Ic(Matrix3r::Zero()); // tensors of inertia; is upper triangular, zeros instead of symmetric elements
 
@@ -385,7 +387,7 @@ void Clump::updatePropertiesNonSpherical(const shared_ptr<Body>& clumpBody, bool
 
 	// begin non-intersecting loop here
 	if(!intersecting){
-		FOREACH(MemberMap::value_type& I, clump->members){
+		for (const auto & I : clump->members){
 			// I.first is Body::id_t, I.second is Se3r of that body
 			shared_ptr<Body> subBody=Body::byId(I.first);
 			State* subState=subBody->state.get();
@@ -437,7 +439,7 @@ void Clump::updatePropertiesNonSpherical(const shared_ptr<Body>& clumpBody, bool
 	clumpBody->setAspherical(state->inertia[0]!=state->inertia[1] || state->inertia[0]!=state->inertia[2]);
 
 	// update subBodySe3s; subtract clump orientation (=apply its inverse first) to subBody's orientation
-	FOREACH(MemberMap::value_type& I, clump->members){
+	for (auto & I : clump->members){
 		// now, I->first is Body::id_t, I->second is Se3r of that body
 		shared_ptr<Body> subBody=Body::byId(I.first);
 		//const shared_ptr<RigidBodyParameters>& subRBP(YADE_PTR_CAST<RigidBodyParameters>(subBody->physicalParameters));
@@ -493,3 +495,6 @@ Matrix3r Clump::inertiaTensorRotate(const Matrix3r& I, const Quaternionr& rot){
 	Matrix3r T=rot.toRotationMatrix();
 	return inertiaTensorRotate(I,T);
 }
+
+} // namespace yade
+
