@@ -12,6 +12,8 @@
 #include "GlobalEngine.hpp"
 #include "Scene.hpp"
 
+namespace yade { // Cannot have #include directive inside.
+
 class Body;
 
 class TimeStepper: public GlobalEngine{
@@ -21,12 +23,15 @@ class TimeStepper: public GlobalEngine{
 		virtual void action() { computeTimeStep(scene);} ;
 		void setActive(bool a, int nb=-1) {active = a; if (nb>0) {timeStepUpdateInterval = (unsigned int)nb;}}
 		
+	// clang-format off
 		YADE_CLASS_BASE_DOC_ATTRS(
 			TimeStepper,GlobalEngine,"Engine defining time-step (fundamental class)",
 			((bool,active,true,,"is the engine active?"))
 			((unsigned int,timeStepUpdateInterval,1,,"dt update interval")));
+	// clang-format on
 };
 
 REGISTER_SERIALIZABLE(TimeStepper);
 
+} // namespace yade
 
