@@ -2,7 +2,9 @@
 """ NOTE
 Needs yade compiled with CGAL feature
 """
+from __future__ import print_function
 
+from builtins import range
 O.engines=[
 	ForceResetter(),
 	InsertionSortCollider([Bo1_Sphere_Aabb(),Bo1_Box_Aabb(),Bo1_Facet_Aabb()]),
@@ -39,6 +41,6 @@ if 1:
 else:
 	#O.run(100,True)
 	O.step()
-	print len(O.interactions)
+	print(len(O.interactions))
 	#O.bodies[2].phys['se3']=[-.6,0,.6,1,0,0,0]
 	#O.step()
