@@ -1,4 +1,8 @@
 # encoding: utf-8
+from builtins import zip
+
+from builtins import range
+from builtins import object
 from PyQt5.QtCore import *
 from PyQt5.QtGui import *
 from PyQt5 import QtGui
@@ -60,7 +64,7 @@ def serializableHref(ser,attr=None,text=None):
 		if not text: text=klass.__name__
 	return makeWrapperHref(text,klass.__name__,attr,static=(attr and getattr(klass,attr)==getattr(ser,attr)))
 
-class AttrEditor():
+class AttrEditor(object):
 	"""Abstract base class handing some aspects common to all attribute editors.
 	Holds exacly one attribute which is updated whenever it changes."""
 	def __init__(self,getter=None,setter=None):
@@ -137,7 +141,7 @@ class AttrEditor_Complex(AttrEditor,QLineEdit):
 		self.first=True
 		val=self.getter()
 		self.grid=QGridLayout(self); self.grid.setSpacing(0); self.grid.setContentsMargins(0,0,0,0)
-		for row,col in itertools.product(range(self.rows),range(self.cols)):
+		for row,col in itertools.product(list(range(self.rows)),list(range(self.cols))):
 			w=QLineEdit('')
 			self.grid.addWidget(w,row,col);
 			w.textEdited.connect(self.isHot)
@@ -145,7 +149,7 @@ class AttrEditor_Complex(AttrEditor,QLineEdit):
 			w.editingFinished.connect(self.update)
 	def refresh(self,force=False):
 		val=self.getter()
-		for row,col in itertools.product(range(self.rows),range(self.cols)):
+		for row,col in itertools.product(list(range(self.rows)),list(range(self.cols))):
 			w=self.grid.itemAtPosition(row,col).widget()
 			if(self.first or force):
 				w.setText(str(val.real if col==0 else val.imag))
@@ -205,7 +209,7 @@ class AttrEditor_Se3(AttrEditor,QFrame):
 		AttrEditor.__init__(self,getter,setter)
 		QFrame.__init__(self,parent)
 		self.grid=QGridLayout(self); self.grid.setSpacing(0);
-		for row,col in itertools.product(range(2),range(5)): # one additional column for vertical line in quaternion
+		for row,col in itertools.product(list(range(2)),list(range(5))): # one additional column for vertical line in quaternion
 			if (row,col)==(0,3): continue
 			if (row,col)==(0,4): self.grid.addWidget(QLabel(u'←<i>pos</i> ↙<i>ori</i>',self),row,col); continue
 			if (row,col)==(1,3):
@@ -251,7 +255,7 @@ class AttrEditor_MatrixX(AttrEditor,QFrame):
 		self.first=True
 		val=self.getter()
 		self.grid=QGridLayout(self); self.grid.setSpacing(0);
-		for row,col in itertools.product(range(self.rows),range(self.cols)):
+		for row,col in itertools.product(list(range(self.rows)),list(range(self.cols))):
 			w=QLineEdit('')
 			self.grid.addWidget(w,row,col);
 			w.textEdited.connect(self.isHot)
@@ -259,7 +263,7 @@ class AttrEditor_MatrixX(AttrEditor,QFrame):
 			w.editingFinished.connect(self.update)
 	def refresh(self,force=False):
 		val=self.getter()
-		for row,col in itertools.product(range(self.rows),range(self.cols)):
+		for row,col in itertools.product(list(range(self.rows)),list(range(self.cols))):
 			w=self.grid.itemAtPosition(row,col).widget()
 			if(self.first or force):
 				w.setText(str(val[self.idxConverter(row,col)]))
@@ -270,7 +274,7 @@ class AttrEditor_MatrixX(AttrEditor,QFrame):
 	def update(self):
 		try:
 			val=self.getter()
-			for row,col in itertools.product(range(self.rows),range(self.cols)):
+			for row,col in itertools.product(list(range(self.rows)),list(range(self.cols))):
 				w=self.grid.itemAtPosition(row,col).widget()
 				if w.isModified(): val[self.idxConverter(row,col)]=float(w.text())
 			logging.debug('setting'+str(val))
@@ -288,20 +292,20 @@ class AttrEditor_MatrixXi(AttrEditor,QFrame):
 		self.idxConverter=idxConverter
 		self.setContentsMargins(0,0,0,0)
 		self.grid=QGridLayout(self); self.grid.setSpacing(0); self.grid.setContentsMargins(0,0,0,0)
-		for row,col in itertools.product(range(self.rows),range(self.cols)):
+		for row,col in itertools.product(list(range(self.rows)),list(range(self.cols))):
 			w=QSpinBox()
 			w.setRange(int(-1e9),int(1e9)); w.setSingleStep(1);
 			self.grid.addWidget(w,row,col);
 		self.refresh() # refresh before connecting signals!
-		for row,col in itertools.product(range(self.rows),range(self.cols)):
+		for row,col in itertools.product(list(range(self.rows)),list(range(self.cols))):
 			self.grid.itemAtPosition(row,col).widget().valueChanged.connect(self.update)
 	def refresh(self):
 		val=self.getter()
-		for row,col in itertools.product(range(self.rows),range(self.cols)):
+		for row,col in itertools.product(list(range(self.rows)),list(range(self.cols))):
 			w=self.grid.itemAtPosition(row,col).widget().setValue(val[self.idxConverter(row,col)])
 	def update(self):
 		val=self.getter(); modified=False
-		for row,col in itertools.product(range(self.rows),range(self.cols)):
+		for row,col in itertools.product(list(range(self.rows)),list(range(self.cols))):
 			w=self.grid.itemAtPosition(row,col).widget()
 			if w.value()!=val[self.idxConverter(row,col)]:
 				modified=True; val[self.idxConverter(row,col)]=w.value()
@@ -333,7 +337,7 @@ class AttrEditor_Matrix3(AttrEditor_MatrixX):
 	def __init__(self,parent,getter,setter):
 		AttrEditor_MatrixX.__init__(self,parent,getter,setter,3,3,lambda r,c:(r,c))
 
-class Se3FakeType: pass
+class Se3FakeType(object): pass
 
 _fundamentalEditorMap={bool:AttrEditor_Bool,str:AttrEditor_Str,int:AttrEditor_Int,float:AttrEditor_Float,complex:AttrEditor_Complex,Quaternion:AttrEditor_Quaternion,Vector2:AttrEditor_Vector2,Vector3:AttrEditor_Vector3,Vector6:AttrEditor_Vector6,Matrix3:AttrEditor_Matrix3,Vector6i:AttrEditor_Vector6i,Vector3i:AttrEditor_Vector3i,Vector2i:AttrEditor_Vector2i,Se3FakeType:AttrEditor_Se3}
 _fundamentalInitValues={bool:True,str:'',int:0,float:0.0,complex:0.0j,Quaternion:Quaternion((0,1,0),0.0),Vector3:Vector3.Zero,Matrix3:Matrix3.Zero,Vector6:Vector6.Zero,Vector6i:Vector6i.Zero,Vector3i:Vector3i.Zero,Vector2i:Vector2i.Zero,Vector2:Vector2.Zero,Se3FakeType:(Vector3.Zero,Quaternion((0,1,0),0.0))}
@@ -359,7 +363,7 @@ class SerializableEditor(QFrame):
 	import collections
 	import logging
 	# each attribute has one entry associated with itself
-	class EntryData:
+	class EntryData(object):
 		def __init__(self,name,T,flags=0):
 			self.name,self.T,self.flags=name,T,flags
 			self.lineNo,self.widget=None,None
@@ -397,7 +401,7 @@ class SerializableEditor(QFrame):
 			m=re.match(regexp,cxxT)
 			return m
 		vecMap={
-			'bool':bool,'int':int,'long':int,'Body::id_t':long,'size_t':long,
+			'bool':bool,'int':int,'long':int,'Body::id_t':int,'size_t':int,
 			'Real':float,'float':float,'double':float,'complex':complex,'std::complex<Real>':complex,
 			'Vector6r':Vector6,'Vector6i':Vector6i,'Vector3i':Vector3i,'Vector2r':Vector2,'Vector2i':Vector2i,
 			'Vector3r':Vector3,'Matrix3r':Matrix3,'Se3r':Se3FakeType,
@@ -406,7 +410,7 @@ class SerializableEditor(QFrame):
 			'IntrCallback':IntrCallback,'BoundFunctor':BoundFunctor,'IGeomFunctor':IGeomFunctor,'IPhysFunctor':IPhysFunctor,'LawFunctor':LawFunctor,'KinematicEngine':KinematicEngine,
 			'GlShapeFunctor':GlShapeFunctor,'GlStateFunctor':GlStateFunctor,'GlIGeomFunctor':GlIGeomFunctor,'GlIPhysFunctor':GlIPhysFunctor,'GlBoundFunctor':GlBoundFunctor,'GlExtraDrawer':GlExtraDrawer
 		}
-		for T,ret in vecMap.items():
+		for T,ret in list(vecMap.items()):
 			if vecTest(T,cxxT):
 				logging.debug("Got type %s from cxx type %s"%(repr(ret),cxxT))
 				return (ret,)
@@ -420,7 +424,7 @@ class SerializableEditor(QFrame):
 			logging.error('TypeError when getting attributes of '+str(self.ser)+',skipping. ')
 			import traceback
 			traceback.print_exc()
-		attrs=self.ser.dict().keys(); attrs.sort()
+		attrs=list(self.ser.dict().keys()); attrs.sort()
 		for attr in attrs:
 			val=getattr(self.ser,attr) # get the value using serattr, as it might be different from what the dictionary provides (e.g. Body.blockedDOFs)
 			t=None
@@ -445,7 +449,7 @@ class SerializableEditor(QFrame):
 			match=re.search(':yattrflags:`\s*([0-9]+)\s*`',doc) # non-empty attribute
 			flags=int(match.group(1)) if match else 0
 			#logging.debug('Attr %s is of type %s'%(attr,((t[0].__name__,) if isinstance(t,tuple) else t.__name__)))
-			self.entries.append(self.EntryData(name=attr,T=t))
+			self.entries.append(self.EntryData(name=attr,T=t,flags=flags))
 			#print("name: "+str(attr)+"\tflag: "+str(flags))
 	def getDocstring(self,attr=None):
 		"If attr is *None*, return docstring of the Serializable itself"
@@ -491,9 +495,11 @@ class SerializableEditor(QFrame):
 			T=entry.T[0]
 			if (issubclass(T,Serializable) or T==Serializable):
 				widget=SeqSerializable(self,getter,setter,T,path=(self.path+'.'+entry.name if self.path else None),shrink=True)
+				if (entry.flags & AttrFlags.readonly): widget.setEnabled(False)
 				return widget
 			if (T in _fundamentalEditorMap):
 				widget=SeqFundamentalEditor(self,getter,setter,T)
+				if (entry.flags & AttrFlags.readonly): widget.setEnabled(False)
 				return widget
 			return None
 		# a serializable
@@ -504,6 +510,7 @@ class SerializableEditor(QFrame):
 			else: path=None
 			widget=SerializableEditor(getattr(self.ser,entry.name),parent=self,showType=self.showType,path=(self.path+'.'+entry.name if self.path else None))
 			widget.setFrameShape(QFrame.Box); widget.setFrameShadow(QFrame.Raised); widget.setLineWidth(1)
+			if (entry.flags & AttrFlags.readonly): widget.setEnabled(False)
 			return widget
 		return None
 	def mkWidgets(self):
@@ -511,7 +518,7 @@ class SerializableEditor(QFrame):
 		grid=QFormLayout()
 		grid.setContentsMargins(2,2,2,2)
 		grid.setVerticalSpacing(0)
-		grid.setLabelAlignment(Qt.AlignRight)
+		grid.setLabelAlignment(Qt.AlignLeft)
 		if self.showType:
 			lab=SerQLabel(self,makeSerializableLabel(self.ser,addr=True,href=True),tooltip=self.getDocstring(),path=self.path)
 			lab.setFrameShape(QFrame.Box); lab.setFrameShadow(QFrame.Sunken); lab.setLineWidth(2); lab.setAlignment(Qt.AlignHCenter); lab.linkActivated.connect(yade.qt.openUrl)
@@ -535,11 +542,11 @@ def makeSerializableLabel(ser,href=False,addr=True,boldHref=True,num=-1,count=-1
 		else: ret+=u'%d. '%num
 	if href: ret+=(u' <b>' if boldHref else u' ')+serializableHref(ser)+(u'</b> ' if boldHref else u' ')
 	else: ret+=ser.__class__.__name__+' '
-	if hasattr(ser,'label') and ser.label: ret+=u' “'+unicode(ser.label)+u'”'
+	if hasattr(ser,'label') and ser.label: ret+=u' “'+str(ser.label)+u'”'
 	# do not show address if there is a label already
 	elif addr:
 		import re
-		ss=unicode(ser); m=re.match(u'<(.*) instance at (0x.*)>',ss)
+		ss=str(ser); m=re.match(u'<(.*) instance at (0x.*)>',ss)
 		if m: ret+=m.group(2)
 		else: logging.warning(u"Serializable converted to str ('%s') does not contain 'instance at 0x…'"%ss)
 	return ret
@@ -668,7 +675,7 @@ class NewFundamentalDialog(QDialog):
 		self.layout.addWidget(self.scroll)
 		self.layout.addWidget(self.buttons)
 		self.setWindowModality(Qt.WindowModal)
-		class FakeObjClass: pass
+		class FakeObjClass(object): pass
 		self.fakeObj=FakeObjClass()
 		self.attrName=attrName
 		Klass=_fundamentalEditorMap.get(typeObj,None)
@@ -743,7 +750,7 @@ class SeqFundamentalEditor(QFrame):
 		field=self.form.itemAt(index,QFormLayout.LabelRole).widget() if index>=0 else None
 		menu=QMenu(self)
 		actNew,actKill,actUp,actDown=[menu.addAction(name) for name in (u'☘ New',u'☠ Remove',u'↑ Up',u'↓ Down')]
-		if index<0: [a.setEnabled(False) for a in actKill,actUp,actDown]
+		if index<0: [a.setEnabled(False) for a in (actKill,actUp,actDown)]
 		if index==len(seq)-1: actDown.setEnabled(False)
 		if index==0: actUp.setEnabled(False)
 		if field: field.setStyleSheet('QWidget { background: green }')
@@ -782,7 +789,7 @@ class SeqFundamentalEditor(QFrame):
 	def rebuild(self):
 		currSeq=self.getter()
 		# clear everything
-		rows=self.form.count()/2
+		rows=int(self.form.count()/2)
 		for row in range(rows):
 			logging.trace('counts',self.form.rowCount(),self.form.count())
 			for wi in self.form.itemAt(row,QFormLayout.FieldRole),self.form.itemAt(row,QFormLayout.LabelRole):
@@ -798,10 +805,10 @@ class SeqFundamentalEditor(QFrame):
 			errMsg.setReadOnly(True); errMsg.setText("Sorry, editing sequences of %s's is not (yet?) implemented."%(self.itemType.__name__))
 			self.form.insertRow(0,'<b>Error</b>',errMsg)
 			return
-		class ItemGetter():
+		class ItemGetter(object):
 			def __init__(self,getter,index): self.getter,self.index=getter,index
 			def __call__(self): return self.getter()[self.index]
-		class ItemSetter():
+		class ItemSetter(object):
 			def __init__(self,getter,setter,index): self.getter,self.setter,self.index=getter,setter,index
 			def __call__(self,val): seq=self.getter(); seq[self.index]=val; self.setter(seq)
 		for i,item in enumerate(currSeq):
