@@ -1,6 +1,6 @@
 /*************************************************************************
 *  Copyright (C) 2012 by François Kneib   francois.kneib@gmail.com       *
-*  Copyright (C) 2012 by Bruno Chareyre   bruno.chareyre@hmg.inpg.fr     *
+*  Copyright (C) 2012 by Bruno Chareyre   bruno.chareyre@grenoble-inp.fr     *
 *  This program is free software; it is licensed under the terms of the  *
 *  GNU General Public License v2 or later. See file LICENSE for details. *
 *************************************************************************/
@@ -31,7 +31,7 @@
 	#include<pkg/common/GLDrawFunctors.hpp>
 #endif
 
-
+namespace yade { // Cannot have #include directive inside.
 
 //!##################	SHAPES   #####################
 
@@ -40,7 +40,9 @@ class GridConnection: public Sphere{
 		virtual ~GridConnection();
 		Real getLength();
 		Vector3r getSegment();
-		void addPFacet(shared_ptr<Body> PF); 
+		void addPFacet(shared_ptr<Body> PF);
+		vector<shared_ptr<Body> > getPFacets() {return pfacetList;}
+	// clang-format off
 	YADE_CLASS_BASE_DOC_ATTRS_CTOR_PY(GridConnection,Sphere,"GridConnection shape (see [Effeindzourou2016]_, [Bourrier2013]_). Component of a grid designed to link two :yref:`GridNodes<GridNode>`. It is highly recommended to use :yref:`yade.gridpfacet.gridConnection` to generate correct :yref:`GridConnections<GridConnection>`.",
 		((shared_ptr<Body> , node1 , ,,"First :yref:`Body` the GridConnection is connected to."))
 		((shared_ptr<Body> , node2 , ,,"Second :yref:`Body` the GridConnection is connected to."))
@@ -48,9 +50,11 @@ class GridConnection: public Sphere{
 		 ((vector<shared_ptr<Body> >,pfacetList,,Attr::hidden,"List of :yref:`PFacet<PFacet>` the GridConnection is connected to."))
 		((Vector3i , cellDist , Vector3i(0,0,0),,"Distance of bodies in cell size units, if using periodic boundary conditions. Note that periodic boundary conditions for GridConnections have not yet been fully implemented.")),
 		createIndex();, /*ctor*/
-				/*py*/			  
-		.def("addPFacet",&GridConnection::addPFacet,(boost::python::arg("Body")),"Add a PFacet to the GridConnection.") 
+		/*py*/  
+		.def("addPFacet",&GridConnection::addPFacet,(boost::python::arg("Body")),"Add a PFacet to the GridConnection.")
+		.def("getPFacets",&GridConnection::getPFacets,"get list of linked PFacets.") 
 	);
+	// clang-format on
 	REGISTER_CLASS_INDEX(GridConnection,Sphere);
 };
 REGISTER_SERIALIZABLE(GridConnection);
@@ -60,7 +64,11 @@ class GridNode: public Sphere{
 	public:
 		virtual ~GridNode();
 		void addConnection(shared_ptr<Body> GC);
-		void addPFacet(shared_ptr<Body> PF); 
+		void addPFacet(shared_ptr<Body> PF);
+		vector<shared_ptr<Body> > getPFacets() {return pfacetList;}
+		vector<shared_ptr<Body> > getConnections() {return ConnList;}
+		
+	// clang-format off
 	YADE_CLASS_BASE_DOC_ATTRS_CTOR_PY(GridNode,Sphere,"GridNode shape, component of a grid.\nTo create a Grid, place the nodes first, they will define the spacial discretisation of it. It is highly recommended to use :yref:`yade.gridpfacet.gridNode` to generate correct :yref:`GridNodes<GridNode>`. Note that the GridNodes should only be in an Interaction with other GridNodes. The Sphere-Grid contact is only handled by the :yref:`GridConnections<GridConnection>`.",
 		((vector<shared_ptr<Body> >,pfacetList,,Attr::hidden,"List of :yref:`PFacets<PFacet>` the GridConnection is connected to."))
 		((vector<shared_ptr<Body> >,ConnList,,Attr::hidden,"List of :yref:`GridConnections<GridConnection>` the GridNode is connected to.")),
@@ -69,7 +77,10 @@ class GridNode: public Sphere{
 		/*py*/
 		.def("addConnection",&GridNode::addConnection,(boost::python::arg("Body")),"Add a GridConnection to the GridNode.")
 		.def("addPFacet",&GridNode::addPFacet,(boost::python::arg("Body")),"Add a PFacet to the GridNode.")
+		.def("getPFacets",&GridNode::getPFacets,"get list of linked :yref:`PFacet`'s.") 
+		.def("getConnections",&GridNode::getConnections,"get list of linked :yref:`GridConnection`'s.") 
 	);
+	// clang-format on
 	REGISTER_CLASS_INDEX(GridNode,Sphere);
 };
 REGISTER_SERIALIZABLE(GridNode);
@@ -90,6 +101,7 @@ class PFacet : public Shape {
 	/// Unit vertice vectors
 	Vector3r vu[3];
 
+	// clang-format off
 	YADE_CLASS_BASE_DOC_ATTRS_CTOR(PFacet,Shape,"PFacet (particle facet) geometry (see [Effeindzourou2016]_, [Effeindzourou2015a]_). It is highly recommended to use the helper functions in :yref:`yade.gridpfacet` (e.g., gridpfacet.pfacetCreator1-4) to generate correct :yref:`PFacet<PFacet>` elements.",
 		((shared_ptr<Body> , node1 , ,,"First :yref:`Body` the Pfacet is connected to."))
 		((shared_ptr<Body> , node2 , ,,"Second :yref:`Body` the Pfacet is connected to."))
@@ -104,6 +116,7 @@ class PFacet : public Shape {
 		,
 		/* ctor */ createIndex();
 	);
+	// clang-format on
 	DECLARE_LOGGER;
 	
 	REGISTER_CLASS_INDEX(PFacet,Shape);
@@ -117,12 +130,14 @@ REGISTER_SERIALIZABLE(PFacet);
 class GridNodeGeom6D: public ScGeom6D {
 	public:
 		virtual ~GridNodeGeom6D();
+	// clang-format off
 	YADE_CLASS_BASE_DOC_ATTRS_INIT_CTOR_PY(GridNodeGeom6D,ScGeom6D,"Geometry of a :yref:`GridNode`-:yref:`GridNode` contact. Inherits almost everything from :yref:`ScGeom6D`.",
 		((shared_ptr<Body>, connectionBody,,,"Reference to the :yref:`GridNode` :yref:`Body` who is linking the two :yref:`GridNodes<GridNode>`.")),
 		/* extra initializers */,
 		/* ctor */ createIndex();,
 		/* py */
 	);
+	// clang-format on
 	REGISTER_CLASS_INDEX(GridNodeGeom6D,ScGeom6D);
 };
 REGISTER_SERIALIZABLE(GridNodeGeom6D);
@@ -133,6 +148,7 @@ class ScGridCoGeom: public ScGeom6D {
 		/// Emulate a sphere whose position is the projection of sphere's center on cylinder sphere, and with motion linearly interpolated between nodes
 		State fictiousState;
 		virtual ~ScGridCoGeom ();
+	// clang-format off
 	YADE_CLASS_BASE_DOC_ATTRS_CTOR(ScGridCoGeom,ScGeom6D,"Geometry of a :yref:`GridConnection`-:yref:`Sphere` contact.",
 		((int,isDuplicate,0,,"this flag is turned true (1) automatically if the contact is shared between two Connections. A duplicated interaction will be skipped once by the constitutive law, so that only one contact at a time is effective. If isDuplicate=2, it means one of the two duplicates has no longer geometric interaction, and should be erased by the constitutive laws."))
 		((int,trueInt,-1,,"Defines the body id of the :yref:`GridConnection` where the contact is real, when :yref:`ScGridCoGeom::isDuplicate`>0."))
@@ -143,6 +159,7 @@ class ScGridCoGeom: public ScGeom6D {
 		((Real,relPos,0,,"position of the contact on the connection (0: node-, 1:node+) |yupdate|")),
 		createIndex(); /*ctor*/
 	);
+	// clang-format on
 	REGISTER_CLASS_INDEX(ScGridCoGeom,ScGeom6D);
 };
 REGISTER_SERIALIZABLE(ScGridCoGeom);
@@ -153,11 +170,13 @@ class GridCoGridCoGeom: public ScGeom {
 		/// Emulate a sphere whose position is the projection of sphere's center on cylinder sphere, and with motion linearly interpolated between nodes
 		State fictiousState1,fictiousState2;
 		virtual ~GridCoGridCoGeom ();
+	// clang-format off
 	YADE_CLASS_BASE_DOC_ATTRS_CTOR(GridCoGridCoGeom,ScGeom,"Geometry of a :yref:`GridConnection`-:yref:`GridConnection` contact.",
 		((Real,relPos1,0,,"position of the contact on the first connection (0: node-, 1:node+) |yupdate|"))
 		((Real,relPos2,0,,"position of the contact on the first connection (0: node-, 1:node+) |yupdate|")),
 		createIndex(); /*ctor*/
 	);
+	// clang-format on
 	REGISTER_CLASS_INDEX(ScGridCoGeom,ScGeom);
 };
 REGISTER_SERIALIZABLE(GridCoGridCoGeom);
@@ -169,10 +188,12 @@ class Ig2_GridNode_GridNode_GridNodeGeom6D: public Ig2_Sphere_Sphere_ScGeom{
 	public:
 		virtual bool go(const shared_ptr<Shape>& cm1, const shared_ptr<Shape>& cm2, const State& state1, const State& state2, const Vector3r& shift2, const bool& force, const shared_ptr<Interaction>& c);
 		virtual bool goReverse(	const shared_ptr<Shape>& cm1, const shared_ptr<Shape>& cm2, const State& state1, const State& state2, const Vector3r& shift2, const bool& force, const shared_ptr<Interaction>& c);
+	// clang-format off
 	YADE_CLASS_BASE_DOC_ATTRS(Ig2_GridNode_GridNode_GridNodeGeom6D,Ig2_Sphere_Sphere_ScGeom,"Create/update a :yref:`GridNodeGeom6D` instance representing the geometry of a contact point between two :yref:`GridNode<GridNode>`, including relative rotations.",
 		((bool,updateRotations,true,,"Precompute relative rotations. Turning this false can speed up simulations when rotations are not needed in constitutive laws (e.g. when spheres are compressed without cohesion and moment in early stage of a triaxial test), but is not foolproof. Change this value only if you know what you are doing."))
 		((bool,creep,false,,"Substract rotational creep from relative rotation. The rotational creep :yref:`ScGeom6D::twistCreep` is a quaternion and has to be updated inside a constitutive law, see for instance :yref:`Law2_ScGeom6D_CohFrictPhys_CohesionMoment`."))
 	);
+	// clang-format on
 	FUNCTOR2D(GridNode,GridNode);
 	// needed for the dispatcher, even if it is symmetric
 	DEFINE_FUNCTOR_ORDER_2D(GridNode,GridNode);
@@ -184,8 +205,10 @@ class Ig2_GridConnection_GridConnection_GridCoGridCoGeom: public IGeomFunctor{
 	public:
 		virtual bool go(const shared_ptr<Shape>& cm1, const shared_ptr<Shape>& cm2, const State& state1, const State& state2, const Vector3r& shift2, const bool& force, const shared_ptr<Interaction>& c);
 		virtual bool goReverse(	const shared_ptr<Shape>& cm1, const shared_ptr<Shape>& cm2, const State& state1, const State& state2, const Vector3r& shift2, const bool& force, const shared_ptr<Interaction>& c);
+	// clang-format off
 	YADE_CLASS_BASE_DOC_ATTRS(Ig2_GridConnection_GridConnection_GridCoGridCoGeom,IGeomFunctor,"Create/update a :yref:`GridCoGridCoGeom` instance representing the geometry of a contact point between two :yref:`GridConnection` , including relative rotations.",
 	);
+	// clang-format on
 	FUNCTOR2D(GridConnection,GridConnection);
 	DEFINE_FUNCTOR_ORDER_2D(GridConnection,GridConnection);
 };
@@ -197,9 +220,11 @@ class Ig2_Sphere_GridConnection_ScGridCoGeom: public IGeomFunctor{
 	public:
 		virtual bool go(const shared_ptr<Shape>& cm1, const shared_ptr<Shape>& cm2, const State& state1, const State& state2, const Vector3r& shift2, const bool& force, const shared_ptr<Interaction>& c);
 		virtual bool goReverse(	const shared_ptr<Shape>& cm1, const shared_ptr<Shape>& cm2, const State& state1, const State& state2, const Vector3r& shift2, const bool& force, const shared_ptr<Interaction>& c);
+	// clang-format off
 	YADE_CLASS_BASE_DOC_ATTRS(Ig2_Sphere_GridConnection_ScGridCoGeom,IGeomFunctor,"Create/update a :yref:`ScGridCoGeom6D` instance representing the geometry of a contact point between a :yref:`GricConnection` and a :yref:`Sphere` including relative rotations.",
 		((Real,interactionDetectionFactor,1,,"Enlarge both radii by this factor (if >1), to permit creation of distant interactions."))
 	);
+	// clang-format on
 	FUNCTOR2D(Sphere,GridConnection);
 	DEFINE_FUNCTOR_ORDER_2D(Sphere,GridConnection);
 };
@@ -212,12 +237,14 @@ REGISTER_SERIALIZABLE(Ig2_Sphere_GridConnection_ScGridCoGeom);
 class Law2_ScGridCoGeom_FrictPhys_CundallStrack: public LawFunctor{
 	public:
 		virtual bool go(shared_ptr<IGeom>& _geom, shared_ptr<IPhys>& _phys, Interaction* I);
+	// clang-format off
 	YADE_CLASS_BASE_DOC_ATTRS_CTOR_PY(Law2_ScGridCoGeom_FrictPhys_CundallStrack,LawFunctor,"Law between a frictional :yref:`GridConnection` and a frictional :yref:`Sphere`. Almost the same than :yref:`Law2_ScGeom_FrictPhys_CundallStrack`, but the force is divided and applied on the two :yref:`GridNodes<GridNode>` only.",
 		((bool,neverErase,false,,"Keep interactions even if particles go away from each other (only in case another constitutive law is in the scene, e.g. :yref:`Law2_ScGeom_CapillaryPhys_Capillarity`)"))
 		((int,plastDissipIx,-1,(Attr::hidden|Attr::noSave),"Index for plastic dissipation (with O.trackEnergy)"))
 		((int,elastPotentialIx,-1,(Attr::hidden|Attr::noSave),"Index for elastic potential energy (with O.trackEnergy)"))
 		,,
 	);
+	// clang-format on
 	FUNCTOR2D(ScGridCoGeom,FrictPhys);
 };
 REGISTER_SERIALIZABLE(Law2_ScGridCoGeom_FrictPhys_CundallStrack);
@@ -225,11 +252,13 @@ REGISTER_SERIALIZABLE(Law2_ScGridCoGeom_FrictPhys_CundallStrack);
 class Law2_ScGridCoGeom_CohFrictPhys_CundallStrack: public LawFunctor{
 	public:
 		virtual bool go(shared_ptr<IGeom>& _geom, shared_ptr<IPhys>& _phys, Interaction* I);
+	// clang-format off
 	YADE_CLASS_BASE_DOC_ATTRS_CTOR_PY(Law2_ScGridCoGeom_CohFrictPhys_CundallStrack,LawFunctor,"Law between a cohesive frictional :yref:`GridConnection` and a cohesive frictional :yref:`Sphere`. Almost the same than :yref:`Law2_ScGeom6D_CohFrictPhys_CohesionMoment`, but THE ROTATIONAL MOMENTS ARE NOT COMPUTED.",
 		((bool,neverErase,false,,"Keep interactions even if particles go away from each other (only in case another constitutive law is in the scene, e.g. :yref:`Law2_ScGeom_CapillaryPhys_Capillarity`)"))
 		((int,plastDissipIx,-1,(Attr::hidden|Attr::noSave),"Index for plastic dissipation (with O.trackEnergy)"))
 		,,
 	);
+	// clang-format on
 	FUNCTOR2D(ScGridCoGeom,CohFrictPhys);
 };
 REGISTER_SERIALIZABLE(Law2_ScGridCoGeom_CohFrictPhys_CundallStrack);
@@ -238,9 +267,11 @@ REGISTER_SERIALIZABLE(Law2_ScGridCoGeom_CohFrictPhys_CundallStrack);
 class Law2_GridCoGridCoGeom_FrictPhys_CundallStrack: public Law2_ScGeom_FrictPhys_CundallStrack{
 	public:
 		virtual bool go(shared_ptr<IGeom>& _geom, shared_ptr<IPhys>& _phys, Interaction* I);
+	// clang-format off
 		YADE_CLASS_BASE_DOC_ATTRS(Law2_GridCoGridCoGeom_FrictPhys_CundallStrack,Law2_ScGeom_FrictPhys_CundallStrack,"Frictional elastic contact law between two :yref:`gridConnection` . See :yref:`Law2_ScGeom_FrictPhys_CundallStrack` for more details.",
 		/*ATTRS*/
 	);
+	// clang-format on
 	FUNCTOR2D(GridCoGridCoGeom,FrictPhys);
 	DECLARE_LOGGER;
 };
@@ -254,9 +285,11 @@ class Bo1_GridConnection_Aabb : public BoundFunctor
 	public :
 		void go(const shared_ptr<Shape>& cm, shared_ptr<Bound>& bv, const Se3r&, const Body*);
 	FUNCTOR1D(GridConnection);
+	// clang-format off
 	YADE_CLASS_BASE_DOC_ATTRS(Bo1_GridConnection_Aabb,BoundFunctor,"Functor creating :yref:`Aabb` from a :yref:`GridConnection`.",
 		((Real,aabbEnlargeFactor,((void)"deactivated",-1),,"Relative enlargement of the bounding box; deactivated if negative."))
 	);
+	// clang-format on
 };
 REGISTER_SERIALIZABLE(Bo1_GridConnection_Aabb);
 
@@ -271,22 +304,18 @@ class Gl1_GridConnection : public GlShapeFunctor{
 	public:
 		virtual void go(const shared_ptr<Shape>&, const shared_ptr<State>&,bool,const GLViewInfo&);
 		void out( Quaternionr q );
+	// clang-format off
 	YADE_CLASS_BASE_DOC_STATICATTRS(Gl1_GridConnection,GlShapeFunctor,"Renders :yref:`Cylinder` object",
 		((bool,wire,false,,"Only show wireframe (controlled by ``glutSlices`` and ``glutStacks``."))
 		((bool,glutNormalize,true,,"Fix normals for non-wire rendering"))
 		((int,glutSlices,8,,"Number of cylinder slices."))
 		((int,glutStacks,4,,"Number of cylinder stacks."))
 	);
+	// clang-format on
 	RENDERS(GridConnection);
 };
 REGISTER_SERIALIZABLE(Gl1_GridConnection);
 #endif
 
-
-
-
-
-
-
-
+} // namespace yade
 
