@@ -25,6 +25,7 @@
 #include <iostream>
 #include <fstream>
 
+namespace yade { // Cannot have #include directive inside.
 
 DT Law2_ScGeom_CapillaryPhys_Capillarity1::dtVbased;
 DT Law2_ScGeom_CapillaryPhys_Capillarity1::dtPbased;
@@ -604,4 +605,6 @@ void Law2_ScGeom_CapillaryPhys_Capillarity1::solver(Real suction,bool reset)
 
 }
 
+} // namespace yade
+
 #endif //LAW2_SCGEOM_CAPILLARYPHYS_Capillarity1
