@@ -4,10 +4,14 @@
 //! To implement potential particles (Houlsby 2009) using sphere
 #include "PotentialParticle.hpp"
 
+namespace yade { // Cannot have #include directive inside.
+
 YADE_PLUGIN((PotentialParticle));
 
 
 PotentialParticle::~PotentialParticle() {
 }
 
+} // namespace yade
+
 #endif // YADE_POTENTIAL_PARTICLES
