@@ -1,6 +1,6 @@
  
 /*************************************************************************
-*  Copyright (C) 2014 by Bruno Chareyre <bruno.chareyre@hmg.inpg.fr>     *
+*  Copyright (C) 2014 by Bruno Chareyre <bruno.chareyre@grenoble-inp.fr>     *
 *                                                                        *
 *  This program is free software; it is licensed under the terms of the  *
 *  GNU General Public License v2 or later. See file LICENSE for details. *
@@ -19,6 +19,8 @@
 #ifdef DFNFLOW
 #include "FlowEngine_DFNFlowEngineT.hpp"
 
+namespace yade { // Cannot have #include directive inside.
+
 class DFNCellInfo : public FlowCellInfo_DFNFlowEngineT
 {
 	public:
@@ -236,7 +238,7 @@ void DFNFlowEngine::interpolateCrack(Tesselation& Tes,Tesselation& NewTes){
 
 void DFNFlowEngine::trickPermeability(RTriangulation::Facet_circulator& facet,Real aperture,RTriangulation::Finite_edges_iterator& ed_it)
 {
-	const RTriangulation::Facet& currentFacet = *facet; /// seems verbose but facet->first was declaring a junk cell and crashing program (https://bugs.launchpad.net/yade/+bug/1666339)
+	const RTriangulation::Facet& currentFacet = *facet; /// seems verbose but facet->first was declaring a junk cell and crashing program (old site, fixed bug https://bugs.launchpad.net/yade/+bug/1666339)
 	const RTriangulation& Tri = solver->T[solver->currentTes].Triangulation();
 	const CellHandle& cell1 = currentFacet.first;
 	const CellHandle& cell2 = currentFacet.first->neighbor(facet->second);
@@ -359,5 +361,7 @@ void DFNFlowEngine::trickPermeability(Solver* flow)
 // 	 }
 // }
 
+} // namespace yade
+
 #endif //DFNFLOW
 #endif //FLOWENGINE
