@@ -8,8 +8,14 @@
 
 Devs: please DO NOT ADD more functions here, it is getting too crowded!
 """
+from __future__ import print_function
 
-import math,random,doctest,geom,numpy
+from future import standard_library
+standard_library.install_aliases()
+
+from builtins import range
+from builtins import object
+import math,random,doctest,numpy
 from yade import *
 from yade.wrapper import *
 try: # use psyco if available
@@ -37,14 +43,14 @@ def saveVars(mark='',loadNow=True,**kw):
 	``loadVars('something')`` and they will be defined in the yade.params.\ *mark* module. The *loadNow* parameter calls :yref:`yade.utils.loadVars`
 	after saving automatically. If 'something' already exists, given variables will be inserted.
 	"""
-	import cPickle
+	import pickle
 	try: 
-		d=cPickle.loads(Omega().tags['pickledPythonVariablesDictionary'+mark])	#load dictionary d
-		for key in kw.keys():
+		d=pickle.loads((Omega().tags['pickledPythonVariablesDictionary'+mark]).encode("ascii"))	#load dictionary d
+		for key in list(kw.keys()):
 			d[key]=kw[key]							#insert new variables into d
 	except KeyError: 
 		d = kw
-	Omega().tags['pickledPythonVariablesDictionary'+mark]=cPickle.dumps(d)
+	Omega().tags['pickledPythonVariablesDictionary'+mark]=pickle.dumps(d,0)	#use protocol 0 as it allows storage in utf8 for python3 and ascii for python2 (boost requirement)
 	if loadNow: loadVars(mark)
 
 
@@ -52,7 +58,7 @@ def loadVars(mark=None):
 	"""Load variables from :yref:`yade.utils.saveVars`, which are saved inside the simulation.
 	If ``mark==None``, all save variables are loaded. Otherwise only those with
 	the mark passed."""
-	import cPickle, types, sys, warnings
+	import pickle, types, sys, warnings
 	def loadOne(d,mark=None):
 		"""Load given dictionary into a synthesized module yade.params.name (or yade.params if *name* is not given). Update yade.params.__all__ as well."""
 		import yade.params
@@ -69,10 +75,10 @@ def loadVars(mark=None):
 			yade.params.__all__+=list(d.keys())
 			yade.params.__dict__.update(d)
 	if mark!=None:
-		d=cPickle.loads(Omega().tags['pickledPythonVariablesDictionary'+mark])
+		d=pickle.loads((Omega().tags['pickledPythonVariablesDictionary'+mark]).encode("ascii"))
 		loadOne(d,mark)
 	else: # load everything one by one
-		for m in Omega().tags.keys():
+		for m in list(Omega().tags.keys()):
 			if m.startswith('pickledPythonVariablesDictionary'):
 				loadVars(m[len('pickledPythonVariableDictionary')+1:])
 
@@ -102,11 +108,12 @@ def typedEngine(name):
 	return [e for e in Omega().engines if e.__class__.__name__==name][0]
 
 def defaultMaterial():
-	"""Return default material, when creating bodies with :yref:`yade.utils.sphere` and friends, material is unspecified and there is no shared material defined yet. By default, this function returns::
+	"""Return default material, when creating bodies with :yref:`yade.utils.sphere` and friends, material is unspecified and there is no shared material defined yet. By default, this function returns
 
-		.. code-block:: python
+	.. code-block:: python
+
+		FrictMat(density=1e3,young=1e7,poisson=.3,frictionAngle=.5,label='defaultMat')
 
-		  FrictMat(density=1e3,young=1e7,poisson=.3,frictionAngle=.5,label='defaultMat')
 	"""
 	return FrictMat(density=1e3,young=1e7,poisson=.3,frictionAngle=.5,label='defaultMat')
 
@@ -237,7 +244,7 @@ def wall(position,axis,sense=0,color=None,material=-1,mask=1):
 	See :yref:`yade.utils.sphere`'s documentation for meaning of other parameters."""
 	b=Body()
 	b.shape=Wall(sense=sense,axis=axis,color=color if color else randomColor())
-	if isinstance(position,(int,long,float)):
+	if isinstance(position,(int,int,float)):
 		pos2=Vector3(0,0,0); pos2[axis]=position
 	else: pos2=position
 	_commonBodySetup(b,0,Vector3(0,0,0),material,pos=pos2,fixed=True)
@@ -266,7 +273,7 @@ def facet(vertices,dynamic=None,fixed=True,wire=True,color=None,highlight=False,
 	return b
 
 
-def tetraPoly(vertices,dynamic=True,fixed=False,wire=True,color=None,highlight=False,noBound=False,material=-1,mask=1,chain=-1):
+def tetraPoly(vertices,fixed=False,wire=True,color=None,highlight=False,noBound=False,material=-1,mask=1,chain=-1):
 	"""Create tetrahedron (actually simple Polyhedra) with given parameters.
 
 	:param [Vector3,Vector3,Vector3,Vector3] vertices: coordinates of vertices in the global coordinate system.
@@ -284,7 +291,7 @@ def tetraPoly(vertices,dynamic=True,fixed=False,wire=True,color=None,highlight=F
 	b.chain=chain
 	return b
 
-def tetra(vertices,strictCheck=True,dynamic=True,fixed=False,wire=True,color=None,highlight=False,noBound=False,material=-1,mask=1,chain=-1):
+def tetra(vertices,strictCheck=True,fixed=False,wire=True,color=None,highlight=False,noBound=False,material=-1,mask=1,chain=-1):
 	"""Create tetrahedron with given parameters.
 
 	:param [Vector3,Vector3,Vector3,Vector3] vertices: coordinates of vertices in the global coordinate system.
@@ -296,7 +303,7 @@ def tetra(vertices,strictCheck=True,dynamic=True,fixed=False,wire=True,color=Non
 	volume = TetrahedronSignedVolume(vertices)
 	if volume < 0:
 		if strictCheck:
-			raise RuntimeError, "tetra: wrong order of vertices"
+			raise RuntimeError("tetra: wrong order of vertices")
 		temp = vertices[3]
 		vertices[3] = vertices[2]
 		vertices[2] = temp
@@ -312,7 +319,7 @@ def tetra(vertices,strictCheck=True,dynamic=True,fixed=False,wire=True,color=Non
 	b.chain = chain
 	return b
 
-def polyhedron(vertices,dynamic=True,fixed=False,wire=True,color=None,highlight=False,noBound=False,material=-1,mask=1,chain=-1):
+def polyhedron(vertices,fixed=False,wire=True,color=None,highlight=False,noBound=False,material=-1,mask=1,chain=-1):
 	"""Create polyhedron with given parameters.
 
 	:param [[Vector3]] vertices: coordinates of vertices in the global coordinate system.
@@ -353,11 +360,12 @@ def facetCylinder(*args,**kw):
 	return geom.facetCylinder(*args,**kw)
 	
 def aabbWalls(extrema=None,thickness=0,oversizeFactor=1.5,**kw):
-	"""Return 6 boxes that will wrap existing packing as walls from all sides;
-	extrema are extremal points of the Aabb of the packing (will be calculated if not specified)
-	thickness is wall thickness (will be 1/10 of the X-dimension if not specified)
-	Walls will be enlarged in their plane by oversizeFactor.
-	returns list of 6 wall Bodies enclosing the packing, in the order minX,maxX,minY,maxY,minZ,maxZ.
+	"""Return 6 boxes that will wrap existing packing as walls from all sides.
+	
+	:param extrema: extremal points of the Aabb of the packing, as a list of two Vector3, or any equivalent type (will be calculated if not specified)
+	:param float thickness: is wall thickness (will be 1/10 of the X-dimension if not specified)
+	:param float oversizeFactor: factor to enlarge walls in their plane.
+	:returns: a list of 6 wall Bodies enclosing the packing, in the order minX,maxX,minY,maxY,minZ,maxZ.
 	"""
 	walls=[]
 	if not extrema: extrema=aabbExtrema()
@@ -400,7 +408,7 @@ def perpendicularArea(axis):
 	return (ext[1][other[0]]-ext[0][other[0]])*(ext[1][other[1]]-ext[0][other[1]])
 
 def fractionalBox(fraction=1.,minMax=None):
-	"""retrurn (min,max) that is the original minMax box (or aabb of the whole simulation if not specified)
+	"""Return (min,max) that is the original minMax box (or aabb of the whole simulation if not specified)
 	linearly scaled around its center to the fraction factor"""
 	if not minMax: minMax=aabbExtrema()
 	half=[.5*(minMax[1][i]-minMax[0][i]) for i in [0,1,2]]
@@ -507,11 +515,11 @@ def makeVideo(frameSpec,out,renameNotOverwrite=True,fps=24,kbps=6000,bps=None):
 	if renameNotOverwrite and os.path.exists(out):
 		i=0
 		while(os.path.exists(out+"~%d"%i)): i+=1
-		os.rename(out,out+"~%d"%i); print "Output file `%s' already existed, old file renamed to `%s'"%(out,out+"~%d"%i)
+		os.rename(out,out+"~%d"%i); print("Output file `%s' already existed, old file renamed to `%s'"%(out,out+"~%d"%i))
 	if isinstance(frameSpec,list) or isinstance(frameSpec,tuple): frameSpec=','.join(frameSpec)
 	for passNo in (1,2):
 		cmd=['mencoder','mf://%s'%frameSpec,'-mf','fps=%d'%int(fps),'-ovc','lavc','-lavcopts','vbitrate=%d:vpass=%d:threads=%d:%s'%(int(kbps),passNo,O.numThreads,'turbo' if passNo==1 else ''),'-o',('/dev/null' if passNo==1 else out)]
-		print 'Pass %d:'%passNo,' '.join(cmd)
+		print('Pass %d:'%passNo,' '.join(cmd))
 		ret=subprocess.call(cmd)
 		if ret!=0: raise RuntimeError("Error when running mencoder.")
 
@@ -642,7 +650,7 @@ def trackPerfomance(updateTime=5):
 				w=s.split()
 				threadsCpu[w[0]]=float(w[8])
 			plot.addData(Iteration=curIter,Iter=curIter,Perfomance=perf,Bodies=len(O.bodies),Interactions=len(O.interactions),**threadsCpu)
-			plot.plots.update({'Iter':threadsCpu.keys()})
+			plot.plots.update({'Iter':list(threadsCpu.keys())})
 			lastTime=time.time();lastIter=O.iter
 
 	thread.start_new_thread(__track_perfomance,(updateTime))
@@ -684,7 +692,7 @@ def _deprecatedUtilsFunction(old,new):
 #		return yade.ymport.stl(*args,**kw)
 
 
-class TableParamReader():
+class TableParamReader(object):
 	"""Class for reading simulation parameters from text file.
 
 Each parameter is represented by one column, each parameter set by one line. Colums are separated by blanks (no quoting).
@@ -722,14 +730,14 @@ This class is used by :yref:`yade.utils.readParamsFromTable`.
 			for i in range(len(headings)):
 				val[headings[i]]=ll[l].split()[i]
 			values[l]=val
-		lines=values.keys(); lines.sort()
+		lines=list(values.keys()); lines.sort()
 		# replace '=' by previous value of the parameter
 		for i,l in enumerate(lines):
-			for j in values[l].keys():
+			for j in list(values[l].keys()):
 				if values[l][j]=='=':
 					try:
 						values[l][j]=values[lines[i-1]][j]
-					except IndexError,KeyError:
+					except IndexError as KeyError:
 						raise RuntimeError("The = specifier on line %d refers to nonexistent value on previous line?"%l)
 		#import pprint; pprint.pprint(headings); pprint.pprint(values)
 		# add descriptions, but if they repeat, append line number as well
@@ -801,7 +809,7 @@ def readParamsFromTable(tableFileLine=None,noTableOk=True,unknownOk=False,**kw):
 	tagsParams=[]
 	# dictParams is what eventually ends up in yade.params.table (default+specified values)
 	dictDefaults,dictParams,dictAssign={},{},{}
-	import os, __builtin__,re,math
+	import os, builtins,re,math
 	if not tableFileLine and ('YADE_BATCH' not in os.environ or os.environ['YADE_BATCH']==''):
 		if not noTableOk: raise EnvironmentError("YADE_BATCH is not defined in the environment")
 		O.tags['line']='l!'
@@ -810,24 +818,24 @@ def readParamsFromTable(tableFileLine=None,noTableOk=True,unknownOk=False,**kw):
 		env=tableFileLine.split(':')
 		tableFile,tableLine=env[0],int(env[1])
 		allTab=TableParamReader(tableFile).paramDict()
-		if not allTab.has_key(tableLine): raise RuntimeError("Table %s doesn't contain valid line number %d"%(tableFile,tableLine))
+		if tableLine not in allTab: raise RuntimeError("Table %s doesn't contain valid line number %d"%(tableFile,tableLine))
 		vv=allTab[tableLine]
 		O.tags['line']='l%d'%tableLine
 		O.tags['description']=vv['description']
 		O.tags['id.d']=O.tags['id']+'.'+O.tags['description']; O.tags['d.id']=O.tags['description']+'.'+O.tags['id']
 		# assign values specified in the table to python vars
 		# !something cols are skipped, those are env vars we don't treat at all (they are contained in description, though)
-		for col in vv.keys():
+		for col in list(vv.keys()):
 			if col=='description' or col[0]=='!': continue
-			if col not in kw.keys() and (not unknownOk): raise NameError("Parameter `%s' has no default value assigned"%col)
+			if col not in list(kw.keys()) and (not unknownOk): raise NameError("Parameter `%s' has no default value assigned"%col)
 			if vv[col]=='*': vv[col]=kw[col] # use default value for * in the table
-			elif col in kw.keys(): kw.pop(col) # remove the var from kw, so that it contains only those that were default at the end of this loop
+			elif col in list(kw.keys()): kw.pop(col) # remove the var from kw, so that it contains only those that were default at the end of this loop
 			#print 'ASSIGN',col,vv[col]
 			tagsParams+=['%s=%s'%(col,vv[col])];
 			dictParams[col]=eval(vv[col],math.__dict__)
 	# assign remaining (default) keys to python vars
 	defaults=[]
-	for k in kw.keys():
+	for k in list(kw.keys()):
 		dictDefaults[k]=kw[k]
 		defaults+=["%s=%s"%(k,kw[k])];
 	O.tags['defaultParams']=",".join(defaults)
@@ -853,12 +861,12 @@ def psd(bins=5, mass=True, mask=-1):
 	minD = 0.0
 	
 	for b in O.bodies:
-		if (isinstance(b.shape,Sphere) and ((mask<0) or ((b.mask&mask)<>0))):
+		if (isinstance(b.shape,Sphere) and ((mask<0) or ((b.mask&mask)!=0))):
 			if ((2*b.shape.radius)	> maxD) : maxD = 2*b.shape.radius
 			if (((2*b.shape.radius)	< minD) or (minD==0.0)): minD = 2*b.shape.radius
 
 	if (minD==maxD):
-		print 'Monodisperse packing with diameter =', minD,'. Not computing psd'
+		print('Monodisperse packing with diameter =', minD,'. Not computing psd')
 		return False       #All particles are having the same size
   
 	binsSizes = numpy.linspace(minD, maxD, bins+1)
@@ -868,7 +876,7 @@ def psd(bins=5, mass=True, mask=-1):
 	binsNumbers = numpy.zeros(bins)
 	
 	for b in O.bodies:
-		if (isinstance(b.shape,Sphere) and ((mask<0) or ((b.mask&mask)<>0))):
+		if (isinstance(b.shape,Sphere) and ((mask<0) or ((b.mask&mask)!=0))):
 			d=2*b.shape.radius
 			
 			basketId = int(math.floor( (d-minD) / deltaBinD ) )
@@ -892,7 +900,7 @@ def psd(bins=5, mass=True, mask=-1):
 			i+=1
 	return binsSizes, binsProc, binsSumCum
 
-class clumpTemplate:
+class clumpTemplate(object):
 	"""Create a clump template by a list of relative radii and a list of relative positions. Both lists must have the same length.
 	
 	:param [float,float,...] relRadii: list of relative radii (minimum length = 2)
